/*
 *
 *
 *    DotGraph.h
 *
 *    Dot-graph class for LEDs attached to digital lines.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2016 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *
 *
 */

#ifndef __KK5JY_DOTGRAPH_H
#define __KK5JY_DOTGRAPH_H

namespace KK5JY {
	//
	//  DotGraph - drive a dot-graph of LEDs on discrete digital pins
	//
	class DotGraph {
		private:
			const uint8_t * const pins;		// pin array
			const uint8_t length;			// length of pin array
			const int8_t inc;				// increment between thresholds
			
		public:
			// ctor - assign pins and number of pins
			DotGraph(const uint8_t * const _pins, uint8_t _length);

			// update the graph; values in range of (-127, 127)
			void update(int8_t newValue) const;

			// clear the graph, turning off all LEDs
			void clear(void) const;

			// run test sequence on LEDs
			void test(void) const;
	};

	//
	//  ctor
	//
	inline DotGraph::DotGraph(const uint8_t * const _pins, uint8_t _length)
		: pins(_pins),
		  length(_length),
		  inc(256 / length) {
		for (uint8_t i = 0; i != length; ++i) {
			pinMode(pins[i], OUTPUT);
			digitalWrite(pins[i], LOW);
		}
	}

	//
	//  test
	//
	inline void DotGraph::test(void) const {
		for (uint8_t i = 0; i != length; ++i) {
			digitalWrite(pins[i], HIGH);
			delay(100);
		}
		for (uint8_t i = 0; i != length; ++i) {
			digitalWrite(pins[i], LOW);
			delay(100);
		}
	}

	//
	//  update
	//
	inline void DotGraph::update(int8_t newValue) const {
		uint8_t j = 0;
		int8_t th = -127 + inc;
		bool found = false;
		for (int8_t j = 0; j != length; ++j, th += inc) {
			if ((!found) && (newValue <= th)) {
				found = true;
				digitalWrite(pins[j], HIGH);
			} else {
				digitalWrite(pins[j], LOW);
			}
		}
		if (!found)
			digitalWrite(pins[length - 1], HIGH);
	}

	//
	//  clear
	//
	inline void DotGraph::clear(void) const {
		for (uint8_t j = 0; j != length; ++j) {
			digitalWrite(pins[j], LOW);
		}
	}
}

#endif
