/*
 *
 *
 *    cfsk.h
 *
 *    FSK demodulator.
 *
 *    Copyright (C) 2016 by Matt Roberts.
 *    License: GNU GPL3 (www.gnu.org)
 *
 *
 */

#ifndef __CFSK_H
#define __CFSK_H

#include <stdint.h>
#include "cavg.h"
#include "ctrthr.h"
#include "sign.h"
#include "fixed.h"
#include "minmax.h"

//
//  Reference:
//     FSK Modulation and Demodulation With the MSP430 Microcontroller (TI)
//     http://www.ti.com/lit/an/slaa037/slaa037.pdf
//

namespace KK5JY {
	namespace DSP {

		//
		//  Demodulator class
		//
		class DelayLineFskDemod {
			public:
				struct Config {
					// center frequency
					double f0;
					// frequency correction (to compensate for clock error, rounding, etc.)
					double corr;
					// sampling frequency
					uint16_t fs;
					// integrator length
					uint16_t int_len;
					// tracker length
					uint16_t trk_len;
					// counter rail
					uint8_t ctr_limit;
					// counter trigger level
					uint8_t ctr_trigger;

					Config() {
						f0 = 1000;
						corr = 0.0;
						fs = 8000;
						int_len = 100;
						trk_len = 1600;
						ctr_limit = 10;
						ctr_trigger = 6;
					}
				};
			public:
				// the exact value of the quarter-wavelength delay line
				const double N;
				// the length of the delay array
				const uint16_t length;
			private:
				// the whole part of N
				const uint16_t idx;
				// the fraction * FACTOR
				const FixedPoint<int16_t> frac;
				// 1 - frac
				const FixedPoint<int16_t> frac_1m;
				// divisor for 8-bit fixed data
				const FixedPoint<int16_t> div8bit;

				// the delay line
				int8_t * const delay;
				// a pointer for walking the delay line
				int8_t * ptr;
				// the integrator - int16 samples, int32 accumulator
				Averager<int16_t, int32_t> integrator;
				// the output hysteresis counter
				CounterThreshold<int8_t, int8_t> ctr;
				// min-max deviation tracking for tuning indicator
				MinMaxTrack<int32_t> devMinMax;
				volatile int32_t devMean;

			public:
				DelayLineFskDemod(const Config &c);

			public:
				// run one sample through the demodulator
				bool run(int8_t sample);

				// return the integrator length
				size_t avgSize() const { return integrator.size(); }

				// return the mean envelope deviation
				int16_t deviation() const { return devMean * div8bit; }

				// return the current output counter value
				int8_t counter() const { return ctr.value(); }
		};

		//
		//  ctor
		//
		inline DelayLineFskDemod::DelayLineFskDemod(const Config &c)
			: N(c.corr + (c.fs / (4.0 * c.f0))),			// the exact value of N
			  length(static_cast<uint16_t>(round(N + 2))),	// history length
			  idx(static_cast<uint16_t>(N)),				// the whole part of N
			  frac(N - static_cast<uint16_t>(N)),			// the fractional part of N
			  frac_1m(1.0 - frac),							// 1 - frac
			  div8bit(1.0 / 128.0),							// divisor for 8-bit data
			  delay(new int8_t[length]),					// the delay array
			  integrator(c.int_len),						// the integrator
			  ctr(c.ctr_limit, c.ctr_trigger),				// output hysteresis counter
			  devMinMax(c.trk_len),							// min/max for deviation tracking
			  devMean(0) {
			// initialize the quadrature delay line
			for (uint16_t i = 0; i != length; ++i) {
				delay[i] = 0;
			}
			ptr = delay;
		}

		//
		//  run
		//
		inline bool DelayLineFskDemod::run(int8_t sample) {
			// fetch the quadrature sample
			int8_t *q = ptr + idx;
			if (q >= (delay + length)) {
				q -= length;
			}
			int8_t q1 = *q;

			// and interpolate, if needed
			q = ptr + idx + 1;
			if (q >= (delay + length)) {
				q -= length;
			}
			int8_t q2 = *q;
			q1 = (q2 * frac) + (q1 * frac_1m);

			// compute the product of the the signal and its quadrature;
			//   then run the integrator on the product
			integrator.add(sample * q1);

			// move the current pointer backwards through the delay line
			*ptr-- = sample;
			if (ptr == (delay - 1)) {
				ptr = delay + length - 1;
			}

			// run deviation tracking
			if (devMinMax.run(integrator.accum())) {
				devMean = devMinMax.mean();
			}

			// run output hysteresis
			return ctr.run(sign(integrator.accum()));
		}
	}
}

#endif
